/*-------------------------------------------------------------------------
 * Filename:      main.h
 * Version:       $Id: main.h,v 1.2 2001/08/06 22:44:52 erikm Exp $
 * Copyright:     Copyright (C) 1999, Jan-Derk Bakker
 * Author:        Jan-Derk Bakker <J.D.Bakker@its.tudelft.nl>
 * Description:   Header file for main.c
 * Created at:    Mon Aug 23 20:00:00 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Modified at:   Sun Oct  3 21:44:01 1999
 *-----------------------------------------------------------------------*/
/*
 * main.h: Header file for main.c
 *
 * Copyright (C) 1999  Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: main.h,v 1.2 2001/08/06 22:44:52 erikm Exp $"

#ifndef BLOB_MAIN_H
#define BLOB_MAIN_H

/* memory start and end */
#define MEMORY_START		(0xc0000000)
#define MEMORY_END		(0xe0000000)


/* where does blob live when we download it */
#if defined SHANNON
#define BLOB_RAM_BASE        (0xc0100000)
#else
#define BLOB_RAM_BASE        (0xc1000000)
#endif

#define BLOB_BLOCK_OFFSET    (0x00000000)


/* where does the kernel live in RAM */
#define KERNEL_RAM_BASE      (0xC0008000)

#define KERNEL_BLOCK_OFFSET  (0x00008000)


/* where does the ramdisk live in RAM */
#if defined ASSABET
#  define RAMDISK_RAM_BASE     (0xC0800000)
#elif defined SHANNON
#  define RAMDISK_RAM_BASE     (0xC8000000)
#else
#  define RAMDISK_RAM_BASE     (0xC0400000)
#endif

#if defined ASSABET
#  define RAMDISK_BLOCK_OFFSET (0x00800000)
#elif defined SHANNON
#  define RAMDISK_BLOCK_OFFSET (0x00200000)
#else
#  define RAMDISK_BLOCK_OFFSET (0x00400000)
#endif

/* where does the command block live in memory */
/* As far as I know all architectures have this at the same address */
#define BOOT_PARAMS	(0xc0000100)

/* the size to which the compressed ramdisk expands */
#define RAMDISK_SIZE	(8 * 1024 * 1024)


#include "types.h"
#include "serial.h"


typedef enum {
	fromFlash = 0,
	fromDownload = 1
} block_source_t;


typedef struct {
	int kernelSize;
	block_source_t kernelType;

	int ramdiskSize;
	block_source_t ramdiskType;

	int blobSize;
	block_source_t blobType;

	u32 blockSize;

	eBauds downloadSpeed;
} blob_status_t;


extern blob_status_t blob_status;


#endif
